@@ -32,6 +32,7 @@ APPLAUNCHSERVICES_REQVER = '>=0.1.7'
 ATOMICWRITES_REQVER = '>=1.2.0'
 CHARDET_REQVER = '>=2.0.0'
 CLOUDPICKLE_REQVER = '>=0.5.0'
+COOKIECUTTER_REQVER = '>=1.6.0'
 DIFF_MATCH_PATCH_REQVER = '>=20181111'
 # None for pynsist install for now
 # (check way to add dist.info/egg.info from packages without wheels available)
@@ -49,19 +50,20 @@ PICKLESHARE_REQVER = '>=0.4'
 PSUTIL_REQVER = '>=5.3'
 PYGMENTS_REQVER = '>=2.0'
 PYLINT_REQVER = '>=1.0'
-PYLS_REQVER = '>=0.36.2'
+PYLS_REQVER = '>=0.36.2;<1.0.0'
 PYLS_BLACK_REQVER = '>=0.4.6'
 PYLS_SPYDER_REQVER = '>=0.3.2'
 PYXDG_REQVER = '>=0.26'
 PYZMQ_REQVER = '>=17'
-QDARKSTYLE_REQVER = '>=2.8;<3.0'
+QDARKSTYLE_REQVER = '>=3.0.2'
+QSTYLIZER_REQVER = '>=0.1.10'
 QTAWESOME_REQVER = '>=0.5.7'
 QTCONSOLE_REQVER = '>=5.0.3'
 QTPY_REQVER = '>=1.5.0'
 RTREE_REQVER = '>=0.8.3'
 SETUPTOOLS_REQVER = '>=39.0.0'
 SPHINX_REQVER = '>=0.6.6'
-SPYDER_KERNELS_REQVER = '>=1.10.2;<1.11.0'
+SPYDER_KERNELS_REQVER = '>=2.0.1;<2.1.0'
 TEXTDISTANCE_REQVER = '>=4.2.0'
 THREE_MERGE_REQVER = '>=0.1.1'
 # None for pynsist install for now
@@ -103,6 +105,10 @@ DESCRIPTIONS = [
      'package_name': "cloudpickle",
      'features': _("Handle communications between kernel and frontend"),
      'required_version': CLOUDPICKLE_REQVER},
+    {'modname': "cookiecutter",
+     'package_name': "cookiecutter",
+     'features': _("Create projects from cookiecutter templates"),
+     'required_version': COOKIECUTTER_REQVER},
     {'modname': "diff_match_patch",
      'package_name': "diff-match-patch",
      'features': _("Compute text file diff changes during edition"),
@@ -192,6 +198,10 @@ DESCRIPTIONS = [
      'package_name': "qdarkstyle",
      'features': _("Dark style for the entire interface"),
      'required_version': QDARKSTYLE_REQVER},
+    {'modname': "qstylizer",
+     'package_name': "qstylizer",
+     'features': _("Customize Qt stylesheets"),
+     'required_version': QSTYLIZER_REQVER},
     {'modname': "qtawesome",
      'package_name': "qtawesome",
      'features': _("Icon theme based on FontAwesome and Material Design icons"),
@@ -232,7 +242,7 @@ DESCRIPTIONS = [
     {'modname': "watchdog",
      'package_name': "watchdog",
      'features': _("Watch file changes on project directories"),
-     'required_version': WATCHDOG_REQVER}
+     'required_version': WATCHDOG_REQVER},
 ]
 
 
@@ -347,7 +357,7 @@ def check(modname):
         if dependency.modname == modname:
             return dependency.check()
     else:
-        raise RuntimeError("Unkwown dependency %s" % modname)
+        raise RuntimeError("Unknown dependency %s" % modname)
 
 
 def status(deps=DEPENDENCIES, linesep=os.linesep):
